#!/usr/bin/env python

"""
Document translator support.

Copyright (C) 2017, 2018, 2019, 2021, 2023 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class Translator:

    "General translator support."

    input_formats = None    # defined by subclasses
    formats = None          # defined by subclasses

    def __init__(self, metadata):

        """
        Initialise the translator with the given 'metadata'.
        """

        self.metadata = metadata

        # Initialisation of any other state.

        self.init()

    def init(self):

        "Initialisation method to be overridden by subclasses."

        pass

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, self.metadata)

    # Translation visitor methods.

    def visit(self, node):

        """
        Visit the 'node', invoking the appropriate serialisation handler, and
        returning the result of the handler.
        """

        return node.visit(self)

    def container(self, container):

        "Visit all nodes in 'container', returning a list of translated nodes."

        nodes = []

        if container.nodes:
            for node in container.nodes:
                n = self.visit(node)
                if n:
                    nodes.append(n)

        return nodes

# vim: tabstop=4 expandtab shiftwidth=4
